import numpy as np
import matplotlib.pyplot as plt
from matplotlib.colors import ListedColormap
from sklearn.cross_validation import train_test_split
from sklearn.preprocessing import StandardScaler
from sklearn.datasets import make_moons, make_circles, make_classification
from sklearn.neighbors import KNeighborsClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.gaussian_process import GaussianProcess
from sklearn.svm import SVC
from sklearn.tree import DecisionTreeClassifier
from sklearn.ensemble import RandomForestClassifier, AdaBoostClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn.lda import LDA
from sklearn.qda import QDA
from sklearn.metrics import classification_report,confusion_matrix
from sklearn import cross_validation
import pickle
import ensemble
import pandas as pd

names = ["Nearest Neighbors", "Linear SVM", "RBF SVM", "Poly SVM","Sigmoid","Decision Tree",
         "Random Forest", "AdaBoost", "Naive Bayes", "LDA", "QDA","LogisticRegression"]
classifiers = [
    KNeighborsClassifier(5),
    SVC(kernel="linear", C=0.025),
    SVC(kernel ="rbf", gamma=2, C=1),
    SVC(kernel="poly",C = 0.2 , gamma = 0.2),
    SVC(kernel="sigmoid"),
    DecisionTreeClassifier(max_depth=7),
    RandomForestClassifier(max_depth=7, n_estimators=10, max_features=1),
    AdaBoostClassifier(),
    GaussianNB(),
    LDA(),
    QDA(),
    LogisticRegression(),
	]
ensemble_classifier = ensemble.EnsembleClassifier(classifiers,voting = 'hard',weights = [6,6,5,9,1,4,6,3,6,6,7,6])

with open('data_100_10_9.pkl','rb') as f:
	pkl_data = pickle.load(f)

figure = plt.figure(figsize=(27, 9))
i = 1

# preprocess dataset, split into training and test part
X_train = pkl_data['training_data']
y_train = pkl_data['training_labels']
X_train = StandardScaler().fit_transform(X_train)
for i in range(len(y_train)):
	print i/100
# X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=.1)
indices = 95,96,97,98,99,195,196,197,198,199,295,296,297,298,299,395,396,397,398,399,495,496,497,498,499,595,596,597,598,599,695,696,697,698,699,795,796,797,798,799,895,896,897,898,899,995,996,997,998,999
X_train = [i for j, i in enumerate(X_train) if j not in indices]
y_train	= [i for j, i in enumerate(y_train) if j not in indices]
X_test = pkl_data['test_data']
X_test = StandardScaler().fit_transform(X_test)
y_test = pkl_data['test_labels']

# iteratecd over classifiers
for name, clf in zip(names, classifiers):
	# ax = plt.subplot(len(datasets), len(classifiers) + 1, i)
	clf.fit(X_train, y_train)
	print np.array(y_train).ravel()
	score = clf.score(X_test, y_test)

	prediction = clf.predict(X_test)
	print "--------------",name,"---------------"
	print (classification_report(prediction,y_test))
	print 
	cm = confusion_matrix(prediction,y_test)
	print cm
	print("CORRECT PREDICTIONS:")
	correct = np.trace(cm)
	print(correct)
	print("TOTAL PREDICTIONS:")
	total = np.sum(cm)
	print(total)
	print("ACCURACY:")
	print(float(correct)/float(total))
	plt.matshow(cm)
	plt.title('Confusion matrix for: '+str(name))
	plt.colorbar()
	plt.ylabel('True label')
	plt.xlabel('Predicted label')
	# plt.show()

ensemble_classifier.fit(X_train,y_train)
prediction = ensemble_classifier.predict(X_test)
print "--------------EnsembleClassifier--------------"
print (classification_report(prediction,y_test))
print 
cm = confusion_matrix(prediction,y_test)
print cm
print("CORRECT PREDICTIONS:")
correct = np.trace(cm)
print(correct)
print("TOTAL PREDICTIONS:")
total = np.sum(cm)
print(total)
print("ACCURACY:")
print(float(correct)/float(total))
plt.matshow(cm)
plt.title('Confusion matrix for: Ensemble Classification')
plt.colorbar()
plt.ylabel('True label')
plt.xlabel('Predicted label')
# plt.show()

