import sys
sys.path.append('..')
from scikits.talkbox.features import mfcc
import numpy as np
import pickle
from pymir import *

GENRES = ["blues", "classical", "country", "disco", "hiphop" ,"jazz", "metal", "pop", "reggae", "rock"]
# GENRES = ["blues", "classical","country","disco"]

def open_audio(filename):
	wav_file = AudioFile.open(filename)
	return wav_file

def comp_mfcc_scikit(data):
	ceps, mspec, spec = mfcc(data)
	num_ceps = ceps.shape[0]
	averaged_mfcc = np.mean(ceps[int(num_ceps*1/10):int(num_ceps*9/10)], axis=0)
	return averaged_mfcc

def comp_mfcc_pymir(data):
	return MFCC.mfcc2(data)

def comp_spectral_centroid(data):
	spectrum = data.spectrum()
	centroid = spectrum.centroid()
	return centroid

def comp_zero_crossings(data):
	return data.zcr()

def comp_chroma(data):
	spectrum = data.spectrum()
	chroma = spectrum.chroma()
	return chroma

def comp_spectral_rolloff(data):
	spectrum = data.spectrum()
	rolloff = spectrum.rolloff()
	return rolloff

def comp_features(data):	
	feature = list(comp_mfcc_scikit(data))
	feature.extend([comp_spectral_centroid(data)])
	feature.extend([comp_zero_crossings(data)])
	feature.extend(comp_chroma(data))
	feature.extend([comp_spectral_rolloff(data)])
	return feature

	
def get_audio_features(genre, num):
	num = str(num)
	if len(num) == 1:
		path = "../genres/"+genre+"/"+genre+"."+"0000"+num+".wav"
	elif len(num) == 2:
		path = "../genres/"+genre+"/"+genre+"."+"000"+num+".wav"

	data = open_audio(path)
	features = comp_features(data)
	return features

def train_data():
	training_data = []
	training_labels = []
	test_data = []
	test_labels = []
	
	for genre in GENRES:
		total_examples = 100
		num_training = 100
		num =0

		while num < num_training:
			print('------------------------------Train_data-----------------------------')
			print("Current genre: "+ str(genre))
			print("Number: "+str(num))
			features = get_audio_features(genre,num)
			training_data.append(features)
			training_labels.append(genre)
			num += 1
		num -= 10
		while num < total_examples:
			print('------------------------------Test_data-----------------------------')
			print("Current genre: "+ str(genre))
			print(" Number: "+str(num))
			features = get_audio_features(genre,num)
			test_data.append(features)
			test_labels.append(genre)
			num+=1

	DATA = {'training_data' : training_data , 'training_labels' : training_labels , 'test_data': test_data, 'test_labels':test_labels}
	return DATA

def create_pickle(feature_data):	
	with open('data_90_10_10.pkl','wb') as output:
		pickle.dump(feature_data, output, pickle.HIGHEST_PROTOCOL)

create_pickle(train_data())