import numpy as np, pickle, struct
import wave,os
from scikits.talkbox.features import mfcc
import matplotlib.pyplot as plt
from sklearn.metrics import confusion_matrix
from sklearn import svm
from sklearn import grid_search
from pybrain.tools.shortcuts import buildNetwork
from pybrain.datasets import SupervisedDataSet
from pybrain.datasets import ClassificationDataSet
from pybrain.supervised.trainers import BackpropTrainer
from pybrain.utilities           import percentError
from pybrain.tools.customxml.networkwriter import NetworkWriter
from pybrain.tools.customxml.networkreader import NetworkReader
from sklearn.preprocessing import StandardScaler
from sklearn.metrics import classification_report
from sklearn.grid_search import GridSearchCV
from sklearn.cross_validation import train_test_split

data_size = int(40000*4100.0/250.0)

GENRES = ["country","classical" ,"metal", "pop", "rock","disco"]
# GENRES = ["blues", "classical", "country", "disco", "jazz", "metal", "pop", "reggae", "rock"]

def audio_proc(filename,data_size):
	wav_file = wave.open(filename,'r')
	data = wav_file.readframes(data_size)
	wav_file.close()
	data = struct.unpack('{n}h'.format(n=data_size), data)
	data = np.array(data)
	return data

def spectral_centroid(data, samplerate=44100):
    magnitudes = np.abs(np.fft.rfft(data)) # magnitudes of positive frequencies
    length = len(data)
    freqs = np.abs(np.fft.fftfreq(length, 1.0/samplerate)[:length//2+1]) # positive frequencies
    return np.sum(magnitudes*freqs) / np.sum(magnitudes) # return weighted mean

def comp_mfcc(data):
	ceps, mspec, spec = mfcc(data)
	num_ceps = ceps.shape[0]
	averaged_mfcc = np.mean(ceps[int(num_ceps*1/10):int(num_ceps*9/10)], axis=0)
	return averaged_mfcc

def get_audio_features(genre,num):
	num = str(num)
	if len(num) == 1:
		path = "../genres/"+genre+"/"+genre+"."+"0000"+num+".wav"
	elif len(num) == 2:
		path = "../genres/"+genre+"/"+genre+"."+"000"+num+".wav"

	# get file data
	data = audio_proc(path,data_size)
	mfcc_features = comp_mfcc(data)
	return mfcc_features

def create_nn(training_data, labels):
	nn_dataset = ClassificationDataSet(13,nb_classes = len(GENRES),class_labels = GENRES)
	for x in range(len(training_data)):
		nn_dataset.addSample(training_data[x],x/90)
	tstdata_temp, trndata_temp = nn_dataset.splitWithProportion( 0.25 )
	
	tstdata = ClassificationDataSet(13, nb_classes = len(GENRES),class_labels = GENRES)
	for n in xrange(0, tstdata_temp.getLength()):
		tstdata.addSample( tstdata_temp.getSample(n)[0], tstdata_temp.getSample(n)[1] )

	trndata = ClassificationDataSet(13, nb_classes = len(GENRES),class_labels = GENRES)
	for n in xrange(0, trndata_temp.getLength()):
		trndata.addSample( trndata_temp.getSample(n)[0], trndata_temp.getSample(n)[1] )
	trndata._convertToOneOfMany( )
	tstdata._convertToOneOfMany( )
	nn = buildNetwork(trndata.indim,13, 22 ,10,trndata.outdim)
	trainer = BackpropTrainer( nn, dataset=trndata, momentum=0.1, learningrate=0.01 , verbose=True, weightdecay=0.01) 
	if  os.path.isfile('genre_net.xml'): 
		nn = NetworkReader.readFrom('genre_net.xml') 
	else:
 		nn = buildNetwork( trndata.indim, 64 , trndata.outdim)

 	NetworkWriter.writeToFile(nn, 'genre_net.xml')
 	trainer.trainEpochs (1000)
	print 'Percent Error on Test dataset: ' , percentError( trainer.testOnClassData (
           dataset=tstdata )
           , tstdata['class'] )
	return nn, tstdata ,trndata

def train_nn(nn,nn_dataset):
	print "Number of training patterns: ", len(nn_dataset)
	print "Input and output dimensions: ", nn_dataset.indim, nn_dataset.outdim
	trainer = BackpropTrainer(nn, nn_dataset)
	trainer.trainEpochs(10)

def train_data():
	training_data = []
	training_labels = []
	test_data = []
	test_labels = []
	
	for genre in GENRES:
		total_examples = 100
		num_training = 90
		num =0

		while num < num_training:
			print('------------------------------Train_data-----------------------------')
			print("Current genre: "+ str(genre))
			print("Number: "+str(num))
			mfcc_features = get_audio_features(genre,num)
			training_data.append(mfcc_features)
			training_labels.append(genre)
			num += 1
		while num < total_examples:
			print('------------------------------Test_data-----------------------------')
			print("Current genre: "+ str(genre))
			print(" Number: "+str(num))
			mfcc_features = get_audio_features(genre,num)
			test_data.append(mfcc_features)
			test_labels.append(genre)
			num+=1

	DATA = {'training_data' : training_data , 'training_labels' : training_labels , 'test_data': test_data, 'test_labels':test_labels}
	return DATA

def create_pickle(tester):	
	with open('data.pkl','wb') as output:
		pickle.dump(tester, output, pickle.HIGHEST_PROTOCOL)

def GridSearchParams(training_data,training_labels,test_data,test_labels):
	tuned_parameters = [{'kernel': ['poly'], 'gamma': [1e-3, 1e-4],'C': [1, 10, 100, 1000]},{'kernel': ['linear'], 'C': [1, 10, 100, 1000]}]
	scores = ['precision', 'recall']

	for score in scores:
		print("# Tuning hyper-parameters for %s" % score)
		print()

		clf = GridSearchCV(svm.SVC(C=1), tuned_parameters, cv=5,scoring='%s_weighted' % score)
		clf.fit(training_data, training_labels)
	print("Best parameters set found on development set:")
	print()
	print(clf.best_params_)
	print()
	print("Grid scores on development set:")
	print()
	for params, mean_score, scores in clf.grid_scores_:
		print("%0.3f (+/-%0.03f) for %r"
			% (mean_score, scores.std() * 2, params))
	print()

	print("Detailed classification report:")
	print()
	print("The model is trained on the full development set.")
	print("The scores are computed on the full evaluation set.")
	print()
	y_true, y_pred = test_labels, clf.predict(test_data)
	print(classification_report(y_true, y_pred))
	print()


# Process the training data once only.
# create_pickle(train_data())

with open('data_100_10_9.pkl','rb') as f:
	pkl_data = pickle.load(f)
print "Done loading feature data."

training_data = pkl_data['training_data']
training_labels = pkl_data['training_labels']
# indices = 96,97,98,99,196,197,198,199,296,297,298,299,396,397,398,399,496,497,498,499,596,597,598,599,696,697,698,699,796,797,798,799,896,897,898,899,996,997,998,999
# indices = 90,91,92,93,94,95,96,97,98,99,190,191,192,193,194,195,196,197,198,199,290,291,292,293,294,295,296,297,298,299,390,391,392,393,394,395,396,397,398,399,490,491,492,493,494,495,496,497,498,499,590,591,592,593,594,595,596,597,598,599,690,691,692,693,694,695,696,697,698,699,790,791,792,793,794,795,796,797,798,799,890,891,892,893,894,895,896,897,898,899,990,991,992,993,994,995,996,997,998,999
# training_data = [i for j, i in enumerate(training_data) if j not in indices]
# training_labels = [i for j, i in enumerate(training_labels) if j not in indices]
# print len(training_data)
test_data = pkl_data['test_data']
test_labels = pkl_data['test_labels']
train_array = np.array(training_data)
# train_array[:,13] = train_array[:,13]/100
# train_array[:,1] = train_array[:,1]/10
# train_array[:,27] = train_array[:,27]/100
# print train_array[0]
# train_array = np.delete(train_array,np.s_[::5],1)
test_array = np.array(test_data)

train,test,train_l,test_l = train_test_split(train_array,training_labels, test_size = 0.9)
# test_array[:,13] = test_array[:,13]/100
# test_array[:,1] = test_array[:,1]/10
# test_array[:,27] = test_array[:,27]/100
# print test_array[0]
# print test_labels
# # test_array = np.delete(test_array,np.s_[::5],1)
# row_sum = train_array.sum(axis=1)
# train_array = train_array/row_sum[:,np.newaxis]
# row_sum = test_array.sum(axis=1)
# test_array = test_array/row_sum[:,np.newaxis]
train = StandardScaler().fit_transform(train)
test =  StandardScaler().fit_transform(test)
#--------------------------------------------------------------------------
# neural_net, tstdata , trndata = create_nn(training_data,labels)
#--------------------------------------------------------------------------
print "Creating SVM classifier"
classifier = svm.SVC(C = 0.2 , kernel = 'poly' , gamma = 0.2)
print "SVM Classifier created"
# SVM_grid_search()
classifier.fit(train , train_l)
print "Training data fit to SVM classifier"
prediction = classifier.predict(test)
# print prediction
# GridSearchParams(train_array,training_labels,test_array,test_labels)
print (classification_report(prediction,test_l))
cm = confusion_matrix(prediction , test_l)
cm = cm.T
print(cm)

print("CORRECT PREDICTIONS:")
correct = np.trace(cm)
print(correct)
print("TOTAL PREDICTIONS:")
total = np.sum(cm)
print(total)
print("ACCURACY:")
print(float(correct)/float(total))

plt.matshow(cm)
plt.title('Confusion matrix')
plt.colorbar()
plt.ylabel('True label')
plt.xlabel('Predicted label')
plt.show()
